@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package:CL-WHO; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-who/who.lisp,v 1.33 2007/05/28 18:31:31 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-who/who.lisp,v 1.35 2007/08/24 08:01:37 edi Exp $
 
 ;;; Copyright (c) 2003-2007, Dr. Edmund Weitz. All rights reserved.
 
@@ -54,6 +54,22 @@
            *empty-tag-end* " />"
            *prologue* "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"))))
 
+(declaim (inline escape-char))
+(defun escape-char (char &key (test *escape-char-p*))
+  (declare (optimize speed))
+  "Returns an escaped version of the character CHAR if CHAR satisfies
+the predicate TEST.  Always returns a string."
+  (if (funcall test char)
+    (case char
+      (#\< "&lt;")
+      (#\> "&gt;")
+      (#\& "&amp;")
+      (#\' "&#039;")
+      (#\" "&quot;")
+      (t (format nil (if (eq *html-mode* :xml) "&#x~x;" "&#~d;")
+                 (char-code char))))
+    (make-string 1 :initial-element char)))
+
 (defun escape-string (string &key (test *escape-char-p*))
   (declare (optimize speed))
   "Escape all characters in STRING which pass TEST. This function is
@@ -91,32 +107,46 @@ for STRING which'll just be returned."
               finally (unless pos
                         (write-sequence string s :start old-pos)))))))
 
-(defun escape-string-minimal (string)
-  "Escape only #\<, #\>, and #\& in STRING."
-  (escape-string string :test #'(lambda (char) (find char "<>&"))))
+(flet ((minimal-escape-char-p (char) (find char "<>&")))
+  (defun escape-char-minimal (char)
+    "Escapes only #\<, #\>, and #\& characters."
+    (escape-char char :test #'minimal-escape-char-p))
+  (defun escape-string-minimal (string)
+    "Escapes only #\<, #\>, and #\& in STRING."
+    (escape-string string :test #'minimal-escape-char-p)))
 
-(defun escape-string-minimal-plus-quotes (string)
-  "Like ESCAPE-STRING-MINIMAL but also escapes quotes."
-  (escape-string string :test #'(lambda (char) (find char "<>&'\""))))
+(flet ((minimal-plus-quotes-escape-char-p (char) (find char "<>&'\"")))
+  (defun escape-char-minimal-plus-quotes (char)
+    "Like ESCAPE-CHAR-MINIMAL but also escapes quotes."
+    (escape-char char :test #'minimal-plus-quotes-escape-char-p))
+  (defun escape-string-minimal-plus-quotes (string)
+    "Like ESCAPE-STRING-MINIMAL but also escapes quotes."
+    (escape-string string :test #'minimal-plus-quotes-escape-char-p)))
 
-(defun escape-string-iso-8859-1 (string)
-  "Escapes all characters in STRING which aren't defined in
-ISO-8859-1."
-  (escape-string string :test #'(lambda (char)
-                                  (or (find char "<>&'\"")
-                                      (> (char-code char) 255)))))
+(flet ((iso-8859-1-escape-char-p (char)
+         (or (find char "<>&'\"")
+             (> (char-code char) 255))))
+  (defun escape-char-iso-8859-1 (char)
+    "Escapes characters that aren't defined in ISO-8859-9."
+    (escape-char char :test #'iso-8859-1-escape-char-p))
+  (defun escape-string-iso-8859-1 (string)
+    "Escapes all characters in STRING which aren't defined in ISO-8859-1."
+    (escape-string string :test #'iso-8859-1-escape-char-p)))
 
 (defun escape-string-iso-8859 (string)
-  "Identical to ESCAPE-STRING-8859-1.  Kept for backward
-compatibility."
+  "Identical to ESCAPE-STRING-8859-1.  Kept for backward compatibility."
   (escape-string-iso-8859-1 string))
 
-(defun escape-string-all (string)
-  "Escapes all characters in STRING which aren't in the 7-bit ASCII
+(flet ((non-7bit-ascii-escape-char-p (char)
+         (or (find char "<>&'\"")
+             (> (char-code char) 127))))
+  (defun escape-char-all (char)
+    "Escapes characters which aren't in the 7-bit ASCII character set."
+    (escape-char char :test #'non-7bit-ascii-escape-char-p))
+  (defun escape-string-all (string)
+    "Escapes all characters in STRING which aren't in the 7-bit ASCII
 character set."
-  (escape-string string :test #'(lambda (char)
-                                  (or (find char "<>&'\"")
-                                      (> (char-code char) 127)))))
+    (escape-string string :test #'non-7bit-ascii-escape-char-p)))
 
 (defun process-tag (sexp body-fn)
   (declare (optimize speed space))
@@ -132,19 +162,19 @@ internally.  Utility function used by TREE-TO-TEMPLATE."
        ;; collect attribute/value pairs into ATTR-LIST and tag body (if
        ;; any) into BODY
        (loop for rest on (cdr sexp) by #'cddr
-	     if (keywordp (first rest))
-	       collect (cons (first rest) (second rest)) into attr
-	     else
-	       do (progn (setq attr-list attr)
-			 (setq body rest)
-			 (return))
-	     finally (setq attr-list attr)))
+             if (keywordp (first rest))
+               collect (cons (first rest) (second rest)) into attr
+             else
+               do (progn (setq attr-list attr)
+                         (setq body rest)
+                         (return))
+             finally (setq attr-list attr)))
       ((listp (first sexp))
        (setq tag (first (first sexp)))
        (loop for rest on (cdr (first sexp)) by #'cddr
-	     if (keywordp (first rest))
+             if (keywordp (first rest))
                collect (cons (first rest) (second rest)) into attr
-	     finally (setq attr-list attr))
+             finally (setq attr-list attr))
        (setq body (cdr sexp))))
     (convert-tag-to-string-list tag attr-list body body-fn)))
 
@@ -155,15 +185,18 @@ forms."
   (declare (optimize speed space))
   (loop with =var= = (gensym)
         with attribute-quote = (string *attribute-quote-char*)
-        for (attr . val) in attr-list
+        for (orig-attr . val) in attr-list
+        for attr = (if *downcase-tokens-p*
+                     (string-downcase orig-attr)
+                     (string orig-attr))
         unless (null val) ;; no attribute at all if VAL is NIL
           if (constantp val)
             if (and (eq *html-mode* :sgml) (eq val t)) ; special case for SGML
-              nconc (list " " (string-downcase attr))
+              nconc (list " " attr)
             else
               nconc (list " "
                           ;; name of attribute
-                          (string-downcase attr)
+                          attr
                           (format nil "=~C" *attribute-quote-char*)
                           ;; value of attribute
                           (cond ((stringp val)
@@ -173,7 +206,7 @@ forms."
                                  val)
                                 ((eq val t)
                                  ;; VAL is T, use attribute's name
-                                 (string-downcase attr))
+                                 attr)
                                 (t
                                  ;; constant form, PRINC it -
                                  ;; EVAL is OK here because of CONSTANTP
@@ -187,18 +220,16 @@ forms."
                                 ((eq ,=var= t) 
                                  ,(case *html-mode*
                                         (:sgml
-                                         `(htm ,(format nil " ~A"
-                                                        (string-downcase attr))))
+                                         `(htm ,(format nil " ~A" attr)))
                                         ;; otherwise default to :xml mode
                                         (t
                                          `(htm ,(format nil " ~A=~C~A~C"
-                                                        (string-downcase attr)
+                                                        attr
                                                         *attribute-quote-char*
-                                                        (string-downcase attr)
+                                                        attr
                                                         *attribute-quote-char*)))))
                                 (t
-                                 (htm ,(format nil " ~A=~C" (string-downcase attr)
-                                               *attribute-quote-char*)
+                                 (htm ,(format nil " ~A=~C" attr *attribute-quote-char*)
                                       (str ,=var=)
                                       ,attribute-quote)))))))
 
@@ -214,7 +245,7 @@ a list of strings or Lisp forms."))
   "The standard method which is not specialized.  The idea is that you
 can use EQL specializers on the first argument."
   (declare (optimize speed space))
-  (let ((tag (if *downcase-tags-p* (string-downcase tag) (string tag))))
+  (let ((tag (if *downcase-tokens-p* (string-downcase tag) (string tag))))
     (nconc
      (if *indent*
        ;; indent by *INDENT* spaces
